#
#           A Simple Linear Regression Model
#                    Yasuharu Okamoto,  2018
#
from readdata import *
import matplotlib.pyplot as plt
import numpy as np
import scipy.stats as scst

print('Input data type...')
print('Data is set in the list RawData -> 1')
print('Data is set in the text file    -> 2')
print('Data is set in the csv file     -> 3')
ck = input('\nYour choice = ')
try:
    if ck == '1':
        RawData, f_out, f_out_nm = ReadData_lst()
    elif ck == '2':
        RawData, f_out, f_out_nm = ReadData_txt()
    elif ck == '3':
        RawData, f_out, f_out_nm = ReadData_csv()
    else:
        print('\nInvalid choice...')
        raise Exception()

        

    f_out.write('Data...\n')
    print('Data...')
    for v in RawData:
        f_out.write('{0}\n'.format(v))
        print(v)

    VarNames = RawData[0]       #   Variable names

    ID = []                     #   Case ID
    y = []                      #   Dependent variable
    X = []                      #   Independent variables
    for v in RawData[1:]:
        ID.append(v[0])
        y.append([v[1]])
        X.append([1, v[2]])
    
    y = np.array(y)
    X = np.array(X)

    b = np.linalg.inv(X.transpose() @ X) @ X.transpose() @ y
    print('b = ', b)
    #
    #       Calculation of the Correlation Coefficient
    #
    X1 = X.transpose()[1]
    vy = []
    for v in y:
        vy.append(v[0])
    y = vy
    r, p = scst.pearsonr(X1, y)
    print('r = ', r)
    R2 = r ** 2
    print('R2 = ', R2)
    #
    #           Drawing the regression line
    #
    min_x = np.amin(X1)
    max_x = np.amax(X1)
    y_left = b[0][0] + b[1][0] * min_x
    y_right = b[0][0] + b[1][0] * max_x
    plt.plot([min_x, max_x],[y_left, y_right], 'b-')

    dis_x = (max_x - min_x) * 0.01
    dis_y = (np.amax(y) - np.amin(y)) * 0.01
    #
    #           Plotting the data points
    #    
    plt.plot(X1, y, 'bo')
    #
    #       Displaying labels and figures
    #
    for vx, vy, name in zip(X1, y, ID):
        plt.text(vx + dis_x, vy + dis_y, name)
    plt.xlabel(VarNames[2])
    plt.ylabel(VarNames[1])
    plt.title('Simple Linear Regression\n' +
              'r = {0:.3}   $R^2$ = {1:.3}'.format(r, R2))

    plt.show()

    f_out.write('\nb0<Constant> = {0:.5}\n'.format(b[0][0]))
    f_out.write('b1<{0}> = {1:.5}\n'.format(VarNames[2], b[1][0]))
    f_out.write('\nr = {0:.3}    R^2 = {1:.3}\n'.format(r, R2))

    f_out.close()
    print('{0} was saved.'.format(f_out_nm))

except Exception as e:
    print('\nException...', e)

